
;	THE 'SERIES GENERATOR' SUBROUTINE
;(Offsets 86, 88 & 8C: 'series_06', 'series_08' & 'series_0C')
;This important subroutine generates the series of Chebyshev polynomials
;which are used to approximate to SIN, ATN, LN & EXP and hence to derive
;the other arithmetic functions which depend on these (COS, TAN, ASN,
;ACS, ** and SQR).
;  The polynomials are generated, for n=1,2,... , by the reccurence relation:
;    T[n+1](z) = 2*z*T[n](z) - T[n-1](z), where T[n](z) is the nth
;Chebyshev polynomial in z.
;  The series in fact generates:
;    T[0],2T[1],2T[2],...,2T[n-1], where n is 6 for SIN, 8 for EXP,
;and 12 decimal, for LN and ATN.
;  The coefficients of the powers of z in these polynomials may be found
;in the 'Handbook of Mathematical Functions' by M.Abramowitz and
;I.A.Stegun (Dover 1965), page 795.
;  BASIC programs showing the generation of each of the four functions
;are given here in the Appendix.
;  In simple terms this subroutine is called with the 'last value' on
;the calc stack, say Z, being a number that bears a simple relationship
;to the argument, say X, when the task is to evaluate, for instance,
;SIN X. The calling subroutine also supplies the list of constants that
;are to be required (six constants for SIN). The SERIES GENERATOR then
;manipulates its data and returns to the calling routine a 'last value'
;that bears a simple relationship to the requested function, for
;instance, SIN X.
;  This subroutine can be considered to have four major parts:
;
;i.	The setting of the loop counter:
;	The calling subroutine passes its parameters in the A reg for use
;	as a counter. The calc is entered at GEN_ENT_1 so that the
;	counter can be set.
L3449:;series_06 ...
	LD	B,A		;Move the parameter to B.
	CALL	L335E;GEN_ENT_1	;In effect a RST 28 inst buts sets
				;the counter.
;ii.	The handling of the 'last value', Z:
;	The loop of the generator requires 2*Z to be placed in mem_0,
;	zero to be placed in mem_2 and the 'last value' to be zero.
;				;calculator stack
	DB	31H;duplicate	;Z,Z
	DB	0FH;addition	;2*Z
	DB	0C0H;st_mem_0	;2*Z			(mem_0 holds 2*Z)
	DB	02H;delete	;-
	DB	0A0H;stk_zero	;0
	DB	0C2H;st_mem_2	;0			(mem_2 holds 0)
;
;iii.	The main loop:
;	The series generated by looping, using BREG as a counter; the
;	constants in the calling subroutine are stacked in turn by
;	calling STK_DATA; the calc is reentered at GEN_ENT_2 so as
;	not to disturb the value of BREG; and the series is built
;	up in form:
;	  B(R) = 2*Z*B(R-1) - B(R-2) + A(R), for R=1,...,n,
;	where A(1),...,A(n) are the constants, supplied by the calling
;	subroutine (SIN, ATN, LN & EXP) and B(0) = 0 = B(-1).
;	  The (R+1)th loop starts with B(R) on the stack and with
;	2*Z, B(R-2) & B(R-1) in mem_0, mem_1 & mem_2 respectively.
L3453:;G_LOOP
	DB	31H;duplicate	;B(R),B(R)
	DB	0E0H;get_mem_0	;B(R),B(R),2*Z
	DB	04H;multiply	;B(R),2*B(R)*Z
	DB	0E2H;get_mem_2	;B(R),2*B(R)*Z,B(R-1)
	DB	0C1H;st_mem_1	;mem_1 holds B(R-1)
	DB	03H;subtract	;B(R),2*B(R)*Z-B(R-1)
	DB	38H;end_calc
;The next constant is placed on the calc stack.
	CALL	L33C6;STK_DATA	;B(R),2*B(R)*Z-B(R-1),A(R+1)
;The Calculator is re-entered without disturbing BREG.
	CALL	L3362;GEN_ENT_2
	DB	0FH;addition	;B(R),2*B(R)*Z-B(R-1)+A(R+1)
	DB	01H;exchange	;2*B(R)*Z-B(R-1)+A(R+1),B(R)
	DB	0C2H;st_mem_2	;mem_2 holds B(R)
	DB	02H;delete	;2*B(R)*Z-B(R-1)+A(R+1)=B(R+1)
	DB	35H;dec_jr_nz	;B(R+1)
	DB	0EEH;to 3453,G_LOOP
;
;iv.	The subtraction of B(n-2):
;	The loop above leaves B(n) on the stack and the required result
;	is given by B(n)-B(n-2).
	DB	0E1H;get_mem_1	;B(n),B(n-2)
	DB	03H;subtract	;B(n)-B(n-2)
	DB	38H;end_calc
	RET			;Finished.

;	THE 'ABSOLUTE MAGNITUDE' FUNCTION
;(Offset 2A: 'abs')
;This subroutine performs its unary operation by ensuring that the sign
;bit of a fl-pnt number is reset.
;  'Small integers' have to be treated separately. Most of the work is
;shared with the 'unary minus' operation.
L346A:;abs
	LD	B,0FFH		;B is set to FF hex.
	JR	L3474;NEG_TEST	;The jump is made into 'unary minus'.

;	THE 'UNARY MINUS' OPERATION
;(Offset 1B: 'negate')
;This subroutine performs its unary operation by changing the sign of
;the 'last value' on the calc stack.
;  Zero is simply returned unchanged. Full five byte fl-pnt numbers have
;their sign bit manipulated so that it ends up reset (for 'abs') or
;changed (for 'negate'). 'Small integers' have their sign byte set to
;zero (for 'abs') or changed (for 'negate').
L346E:;NEGATE
	CALL	L34E9;TEST_ZERO	;If the number is zero, the subr returns
	RET	C		;leaving 00 00 00 00 00 unchanged.
	LD	B,0		;B is set to 00 hex for 'negate'.
;'ABS' enters here.
L3474:;NEG_TEST
	LD	A,(HL)		;If the first byte is zero, the
	AND	A		;jump is made to deal with a
	JR	Z,L3483;INT_CASE;'small integer'.
	INC	HL		;Point to the second byte.
	LD	A,B		;Get FF for 'abs', 00 for 'negate'.
	AND	80H		;Now 80 for 'abs', 00 for 'negate'.
	OR	(HL)		;This sets bit 7 for 'abs', but changes
				;nothing for 'negate'.
	RLA			;Now bit 7 is changed, leading to
	CCF			;bit 7 of byte 2 reset for 'abs',
	RRA		        ;and simply changed for 'negate'.
	LD	(HL),A		;The new second byte is stored.
	DEC	HL		;HL points to the first byte again.
	RET			;Finished.

;The 'integer case' does a similar operation with the sign byte.
L3483:;INT_CASE
	PUSH	DE		;Save STKEND in DE.
	PUSH	HL		;Save pointer to the number in HL.
	CALL	L2D7F;INT_FETCH	;Fetch the sign in C, the number in DE.
	POP	HL		;Restore the pnt to the number in HL.
	LD	A,B		;Get FF for 'abs', 00 for 'negate'.
	OR	C		;Now FF for 'abs', no change for 'negate'.
	CPL			;Now 00 for 'abs', and a changed byte
	LD	C,A		;for 'negate'; store it in C.
	CALL	L2D8E;INT_STORE	;Store result on the stack.
	POP	DE		;Return STKEND to DE.
	RET			;Finished.

;	THE 'SIGNUM' FUNCTION
;(Offset 29: 'sign')
;This subroutine handles the function SGN X and therefore returns a 'last
;value' of 1 if X positive, zero if X is zero and -1 if X is negative.
L3492:;sgn
	CALL	L34E9;TEST_ZERO	;If X is zero, just return with zero
	RET	C		;as the 'last value'.
	PUSH	DE		;Save the pnt to STKEND.
	LD	DE,1		;Store 1 in DE.
	INC	HL		;Point to the second byte of X.
	RL	(HL)		;Rotate bit 7 into the carry flag.
	DEC	HL		;Point to the destination again.
	SBC	A,A		;Set C to zero for positive X and
	LD	C,A		;to FF hex for negative X.
	CALL	L2D8E;INT_STORE	;Stack 1 or -1 as required.
	POP	DE		;Restore the pointer to STKEND.
	RET			;Finished.

;	THE 'IN' FUNCTION
;(Offset 2C: 'in')
;This subroutine handles the function IN X. It inputs at processor level
;from port X, loading BC with X and performing the instruction IN A,(C).
L34A5:;in
	CALL	L1E99;FIND_INT2	;The 'last value', X, is compressed
				;into BC.
	IN	A,(C)		;The signal is received.
	JR	L34B0;IN_PK_STK

;	THE 'PEEK' FUNCTION
;(Offset 2B: 'peek')
;This subroutine handles the function PEEK X. The 'last value' is
;unstacked by calling FIND_INT2 and replaced by the value of the
;contents of the required location.
L34AC:;peek
	CALL	L1E99;FIND_INT2	;Evaluate the 'last value', rounded
				;to the nearest integer; test that it
				;is in range and return it in BC.
	LD	A,(BC)		;Fetch the required byte.
L34B0:;IN_PK_STK
	JP	L2D28;STACK_A	;Exit by jumping to STACK_A.

;	THE 'USR' FUNCTION
;Offset 2D: 'usr_no')
;This subroutine ('USR number' as distinct from 'USR string') handles
;the function USR X, where X is a number. The value of X is obtained in BC,
;a return addr is stacked and the machine code is executed from location X.
L34B3:;usr_no
	CALL	L1E99;FIND_INT2	;Evaluate the 'last value', rounded to
				;the nearest int; test that it is in range
				;and return it in BC.
	LD	HL,L2D2B	;Make the return addr be that
	PUSH	HL		;of the subroutine STACK_BC.
	PUSH	BC		;Make an indirect jump to the
	RET			;required location.
;Note: It is interesting that the IY reg pair is reinitialised when the
;return to STACK_BC has been made, but the important H'L' that holds the
;next literal pnt is not restored should it have been disturbed. For a
;successful return to BASIC, H'L' must on exit from the machine code
;contain the addr in SCANNING of the 'end_calc' instruction, 2758 hex.

;	THE 'USR STRING' FUNCTION
;(Offset 19: 'usr-$')
;This subroutine handles the function USR X$, where X$ is a string.
;The subroutine returns in BC the addr of the bit pattern for the
;user-defined graphic corresponding to X$. It reports error A if X$
;is not a single letter between a and u or a user-defined graphic.
L34BC:;usr_$
	CALL	L2BF1;STK_FETCH	;Fetch the parameters of the string X$.
	DEC	BC		;Decrease the length by 1 to test it.
	LD	A,B		;If the length was not 1, then
	OR	C		;jump to give error report A.
	JR	NZ,L34E7;REPORT_A
	LD	A,(DE)		;Fetch the single code of the string.
	CALL	L2C8D;ALPHA	;Does it denote a letter?
	JR	C,L34D3;USR_RANGE ;If so, jump to gets its addr.
	SUB	90H		;Reduce range for actual user-defined
				;graphics to 0-20 decimal.
	JR	C,L34E7;REPORT_A;Give report A if out of range.
	CP	15H		;Test the range again.
	JR	NC,L34E7;REPORT_A ;Give report A if out of range.
	INC	A		;Make range of user-defined graphics
				;1 to 21 decimal, as for a to u.
L34D3:;USR_RANGE
	DEC	A		;Now make the range 0 to 20 decimal
	ADD	A,A		;in each case. Multiply by 8 to get
	ADD	A,A		;an offset for the addr.
	ADD	A,A
	CP	0A8H		;Test the range of the offset.
	JR	NC,L34E7;REPORT_A ;Give report A if out of range.
	LD	BC,(L5C7B);UDG	;Fetch the addr of the 1st user-defined
				;graphic in BC.
	ADD	A,C		;Add C to the offset.
	LD	C,A		;Store the result back in C.
	JR	NC,L34E4;USR_STACK ;Jump if there is no carry.
	INC	B		;Increment B to complete the addr.
L34E4:
	JP	L2D2B;STACK_BC	;Jump to stack the addr.

;Report A - Invalid argument.
L34E7:;REPORT_A
	RST	8;ERROR_1	;Call the error handling routine.
	DB	9

;	THE 'TEST_ZERO' SUBROUTINE
;This subroutine is called at least nine times to test whether a fl-pnt
;number is zero. This test requires that the first four bytes of the
;number should each be zero. The subroutine returns with the carry flag
;set if the number was in fact zero.
L34E9:;TEST_ZERO
	PUSH	HL		;Save HL on the stack.
	PUSH	BC		;Save BC on the stack.
	LD	B,A		;Save the value of A in B.
	LD	A,(HL)		;Get the 1st byte.
	INC	HL		;Point to the 2nd byte.
	OR	(HL)		;OR 1st byte with 2nd.
	INC	HL		;Point to the 3rd byte.
	OR	(HL)		;OR the result with the 3rd byte.
	INC	HL		;Point to the 4th byte.
	OR	(HL)		;OR the result with the 4th byte.
	LD	A,B		;Restore the original value of A.
	POP	BC		;And of BC.
	POP	HL		;Restore the pointer to the number in HL.
	RET	NZ		;Return with carry reset if any of the
				;four bytes was non-zero.
	SCF			;Set the carry flag to indicate that
	RET			;the number was zero, and return.

;	THE 'GREATER THAN ZERO' OPERATION
;(Offset 37: 'greater-0')
;This subroutine returns a 'last value' of one if the present 'last value'
;is greater than zero and zero otherwise. It is also used by other
;subroutines to 'jump on plus'.
L34F9:;GREATER_0
	CALL	L34E9;TEST_ZERO	;Is the 'last value' zero?
	RET	C		;If so, return.
	LD	A,0FFH		;Jump forward to LESS THAN ZERO but
	JR	L3507;SIGN_TO_C	;signal the opposite action is needed.

;	THE 'NOT' FUNCTION
;(Offset 30: 'not')
;This subroutine returns a 'last value' of one if the present 'last value'
;is zero and zero otherwise. It is also used by other subroutines to 'jump
;on zero'.
L3501:;NOT
	CALL	L34E9;TEST_ZERO	;The carry flag will be set only if the
				;'last value' is zero; this gives the
	JR	L350B;FP_0/1	;correct result. Jump forward.

;	THE 'LESS THAN ZERO' OPERATION
;(Offset 36: 'less_0')
;This subroutine returns a 'last value' of one if the present 'last value'
;is less than zero and zero  otherwise. It is also used by other subroutines
;to 'jump on minus'.
L3506:;less_0
	XOR	A		;Clear the A reg.
L3507:;SIGN_TO_C
	INC	HL		;Point to the sign byte.
	XOR	(HL)		;The carry is reset for a positive number
	DEC	HL		;and set for a negative number; when
	RLCA			;entered from GREATER_0 the opposite sign
				;goes to the carry.

;	THE 'ZERO OR ONE' SUBROUTINE
;This subroutine sets the 'last value' to zero if the carry flag is reset
;and to one if it is set. When called from 'E_TO_FP' however it creates the
;zero or one not on the stack but in mem_0.
L350B:;FP_0/1
	PUSH	HL		;Save the result pointer.
	LD	A,0		;Clear A without disturbing the carry.
	LD	(HL),A		;Set the 1st byte to zero.
	INC	HL		;Point to the 2nd byte.
	LD	(HL),A		;Set the 2nd byte to zero.
	INC	HL		;Point to the 3rd byte.
	RLA			;Rotate the carry into A, making A one if
				;the carry was set, but zero if the carry
				;was reset.
	LD	(HL),A		;Set the 3rd byte to one or zero.
	RRA			;Ensure that A is zero again.
	INC	HL		;Point to the 4th byte.
	LD	(HL),A		;Set the 4th byte to zero.
	INC	HL		;Point to the 5th byte.
	LD	(HL),A		;Set the 5th byte to zero.
	POP	HL		;Restore the result pointer.
	RET			;Finished.

;	THE 'OR' OPERATION
;(Offset 07: 'or')
;This subroutine performs the binary operation 'X OR Y' and returns X if Y is
;zero and the value 1 otherwise.
L351B:;or
	EX	DE,HL		;Point HL at Y, the second number.
	CALL	L34E9;TEST_ZERO	;Test whether Y is zero.
	EX	DE,HL		;Restore the pointers.
	RET	C		;Return if Y was zero; X is now the
				;'last value'.
	SCF			;Set the carry flag and jump back to set
	JR	L350B;FP_0/1	;the 'last value' to 1.

;       THE 'NUMBER AND NUMBER' OPERATION
;(Offset 08: 'no_&_no')
;This subroutine performs the binary operation 'X AND Y' and returns X if
;Y is non-zero and the value zero otherwise.
L3524:;no_&_no
	EX	DE,HL		;Point HL at Y, DE at X.
	CALL	L34E9;TEST_ZERO	;Test whether Y is zero.
	EX	DE,HL		;Swap the pntrs back.
	RET	NC		;Return with X as the 'last value'
				;if Y was non-zero.
	AND	A		;Reset the carry flag and jump back
	JR	L350B;FP_0/1	;to set the 'last value' to zero.

;	THE 'STRING AND NUMBER' OPERATION
;(Offset 10: 'str_&_no')
;This subroutine performs the binary operation 'X$ AND Y' and return X$ if
;Y is non-zero and a null string otherwise.
L352D:;str_&_no
	EX	DE,HL		;Point HL at Y, DE at X$.
	CALL	L34E9;TEST_ZERO	;Test whether Y is zero.
	EX	DE,HL		;Swap the pntrs back.
	RET	NC		;Return with X$ as the 'last value'
				;if Y was non-zero.
	PUSH	DE		;Save the pntr to the number.
	DEC	DE		;Point to the 5th byte of the string
				;parameters, i.e. length_high.
	XOR	A		;Clear the A reg.
	LD	(DE),A		;Length_high is now set to zero.
	DEC	DE		;Point to length_low.
	LD	(DE),A		;Length_low is now set to zero.
	POP	DE		;Return with the string parameters
	RET			;being the 'last value'.

;	THE 'COMPARISON' OPERATIONS
;(Offsets 09 to 0E & 11 to 16: 'no_l_eql', 'no_gr_eq', 'nos_neql',
;	'no_grtr', 'no_less', 'nos_eql', 'str_l_eql', 'str_gr_eq',
;	'strs_neql', 'str_grtr', 'str_less' & 'strs_eql')
;This subroutine is used to perform the twelve possible comparison
;operations. The single operation offset is present in the B reg at
;the start of the subroutine.
L353B:;no_l_eql ...
	LD	A,B		;The single offset goes to the A reg.
	SUB	8		;The range is now 01-06 & 09-0E.
	BIT	2,A		;This range is changed to: 00-02,
	JR	NZ,L3543;EX_OR_NOT ;04-06, 08-0A & 0C-0E.
	DEC	A
L3543:;EX_OR_NOT
	RRCA			;Then reduced to 00-07 with carry set
			        ;for 'greater than or equal to' & 'less
				;than'; the operations with carry set are
	JR	NC,L354E;NU_OR_STR ;then treated as their
	PUSH	AF		;complementary operation once
	PUSH	HL		;the values have been exchanged.
	CALL	L343C;EXCHANGE
	POP	DE
	EX	DE,HL
	POP	AF
L354E:;NU_OR_STR
	BIT	2,A		;The numerical comparisons are now
	JR	NZ,L3559;STRINGS ;separated from the string comparisons
				;by testing bit 2.
	RRCA			;The numerical operations now have the
				;range 00-01 with carry set for 'equal'
				;and 'not equal'.
	PUSH	AF		;Save the offset.
	CALL	L300F;SUBTRACT  ;The numbers are subtracted for
	JR	L358C;END_TESTS	;the final tests.
L3559:;STRINGS
	RRCA			;The strings comparisons now have the
				;range 02-03 with carry set for 'equal'
				;and 'not equal'.
	PUSH	AF		;Save the offset.
	CALL	L2BF1;STK_FETCH	;The lengths and starting addresses
	PUSH	DE		;of the strings are fetched from
	PUSH	BC		;the calculator stack.
	CALL	L2BF1;STK_FETCH
	POP	HL		;The length of the second string.
L3564:;BYTE_COMP
	LD	A,H
	OR	L
	EX	(SP),HL
	LD	A,B
	JR	NZ,L3575;SEC_PLUS ;Jump unless the second string
	OR	C		;is null.
L356B:;SECND_LOW
	POP	BC		;Here the second string is either
				;null or less than the first.
	JR	Z,L3572;BOTH_NULL
	POP	AF
	CCF			;The carry is complemented to
	JR	L3588;STR_TEST	;give the correct test results.
L3572:;BOTH_NULL
	POP	AF		;Here the carry is used as it
	JR	L3588		;stands.
L3575:;SEC_PLUS
	OR	C
	JR	Z,L3585;FRST_LESS ;The first string is now null,
				;the second not.
	LD	A,(DE)		;Neither string is null, so their
	SUB	(HL)		;next bytes are compared.
	JR	C,L3585;FRST_LESS ;The first byte is less.
	JR	NZ,L356B;SECND_LOW ;The second byte is less.
	DEC	BC		;The bytes are equal; so the
	INC	DE		;lengths are decremented and a
	INC	HL		;jump is made to BYTE_COMP
	EX	(SP),HL		;to compare the next bytes of
	DEC	HL		;the reduced strings.
	JR	L3564;BYTE_COMP
L3585:;FRST_LESS
	POP	BC
	POP	AF
	AND	A		;The carry is cleared here for the
				;correct test results.
L3588:;STR_TEST
	PUSH	AF		;For the string tests, a zero is
	RST	28H;FP_CALC	;put on to the calculator stack.
	DB	0A0H;stk_zero
	DB	38H;end_calc
L358C:;END_TESTS
	POP	AF		;These three tests, called as
	PUSH	AF		;needed, give the correct results
	CALL	C,L3501;NOT	;for all twelwe comparisons. The
	POP	AF		;initial carry is set for 'not equal'
	PUSH	AF		;and 'equal', and the final carry
	CALL	NC,L34F9;GREATER_0 ;is set for 'greater than', 'less
	POP	AF		;than' and 'equal'.
	RRCA
	CALL	NC,L3501;NOT
	RET			;Finished.
;
;	THE 'STRING CONCATENATION' OPERATION
;(Offset 17: 'strs_add')
;This subroutine performs the binary operation 'A$+B$'. The parameters
;for these strings are fetched and the total length found. Sufficient
;room to hold both the strings is made available in the work space and
;the strings are copied over. The result of this subroutine is therefore
;to produce a temporary variable A$+B$ that resides in the work space.
L359C:;strs_add
	CALL	L2BF1;STK_FETCH	;The parameters of the second
	PUSH	DE		;string are fetched and saved.
	PUSH	BC
	CALL	L2BF1;STK_FETCH	;The parameters of the first
				;string are fetched and saved.
	POP	HL
	PUSH	HL		;The lengths are now in HL and BC.
	PUSH	DE		;The parameters of the first string
	PUSH	BC		;are saved.
	ADD	HL,BC		;The total length of the two
	LD	B,H		;strings is calculated and passed
	LD	C,L		;to BC.
	RST	30H;BC_SPACES	;Sufficient room is made available.
	CALL	L2AB2;STK_STORE	;The parameters of the new string are
				;passed to the calculator stack.
	POP	BC		;The parameters of the first string
	POP	HL		;are retrieved and the string
	LD	A,B		;copied to the work space
	OR	C		;as long as it is not a null string.
	JR	Z,L35B7;OTHER_STR
	LDIR
L35B7:;OTHER_STR
	POP	BC		;Exactly the same procedure is
	POP	HL		;followed for the second string
	LD	A,B		;thereby giving 'A$+B$'.
	OR	C
	JR	Z,L35BF;STK_PNTRS
	LDIR

;	THE 'STK_PNTRS' SUBROUTINE
;This subroutine resets the HL reg pair to point to the 1st byte of the
;'last value', i.e. STKEND_5, and the DE reg pair to point one-past the
;'last value', i.e. STKEND.

L35BF:;STK_PNTRS
	LD	HL,(L5C65);STKEND ;Fetch the current value of STKEND.
	LD	DE,0FFFBh	;Set DE to -5, twos complement.
	PUSH	HL		;Stack the value for STKEND.
	ADD	HL,DE		;Calculate STKEND-5.
	POP	DE		;DE now holds STKEND and HL
	RET			;holds STKEND-5.

;	THE 'CHR$' FUNCTION
;(Offset 2F: 'chr$')
;This subroutine handles the function CHR$ X and creates a single char
;string in the work space.
L35C9:;chr$
	CALL	L2DD5;FP_TO_A	;The 'last value' is compressed
				;into the A reg.
	JR	C,L35DC;REPORT_B ;Give the error report if X was
				;greater than 255 dec, or
	JR	NZ,L35DC;REPORT_B ;X was a negative number.
	PUSH	AF		;Save the compressed value of X.
	LD	BC,1		;Make one space available in the
	RST	30H;BC_SPACES	;work space.
	POP	AF		;Fetch the value.
	LD	(DE),A		;Copy the value to the work space.
	CALL	L2AB2;STK_STORE	;Pass the parameters of the new string
				;to the calc stack.
	EX	DE,HL		;Reset the pointers.
	RET			;Finished.

;	REPORT_B - Integer out of range
L35DC:;REPORT_B
	RST	8;ERROR_1	;Call the error handling routine.
	DB	0AH

;	THE 'VAL' AND 'VAL$' FUNCTION
;(Offsets 1D: 'val' and 18: 'val$')
;This subroutine handles the functions VAL X$ and VAL$ X$. When handling
;VAL X$, it return a 'last value' that is the result of evaluating the
;string (without its bounding quotes) as a numerical expression. When
;handling VAL$ X$, it evaluates X$ (without its bounding quotes) as a
;string expression, and returns the params of that string expression
;as a 'last value' on the calc stack.
L35DE:;val (val$)
	LD	HL,(L5C5D);CH_ADD ;The curr value of CH_ADD is
	PUSH	HL		;preserved on the machine stack.
	LD	A,B		;The 'offset' for 'val' or 'val$'
				;must be in the B reg; it is now
				;copied to A.
	ADD	A,0E3H		;Produce 0 and carry set for 'val',
				;0FBh and carry reset for 'val$'.
	SBC	A,A		;Produce 0FFh (bit 6 therefore set)
				;for 'val', but 0 (bit 7 reset) for
				;'val$'.
	PUSH	AF		;Save this 'flag' on the machine stack.
	CALL	L2BF1;STK_FETCH	;The params of the string are fetched;
	PUSH	DE		;the starting addr is saved; one byte
	INC	BC		;is added to the length and room made
	RST	30H;BC_SPACES	;available for the string (+1) in the
				;work space.
	POP	HL		;The starting addr of the string goes
				;to HL as a source addr.
	LD	(L5C5D),DE;CH_ADD ;The pointer to the 1st new space goes
	PUSH	DE		;to CH_ADD and to the machine stack.
	LDIR			;The string is copied to the work space,
				;together with an extra byte.
	EX	DE,HL		;Switch the pntrs.
	DEC	HL		;The extra byte is replaced by a
	LD	(HL),0DH	;'carriage return' char.
	RES	7,(IY+01H);FLAGS ;The syntax flag is reset and the
	CALL	L24FB;SCANNING	;string is scanned for correct syntax.
	RST	18H;GET_CHAR	;The char after the string is fetched.
	CP	0Dh		;A check is made that the end of the
				;expression has been reached.
	JR	NZ,L360C;V_RPORT_C ;If not, the error is reported.
	POP	HL		;The starting addr of the string
				;is fetched.
	POP	AF		;The flag for 'val/val$' is fetched
	XOR	(IY+01H);FLAGS	;and bit 6 is compared with bit 6 of
	AND	40h		;the result of the syntax scan.
L360C:;V_RPORT_C
	JP	NZ,L1C8A;REPORT_C ;Report the error if they don't match.
	LD	(L5C5D),HL;CH_ADD ;Start addr to CH_ADD again.
	SET	7,(IY+01H);FLAGS ;The flag is set for line execution.
	CALL	L24FB;SCANNING	;The string is treated as a 'next
				;expression' and a 'last value' produced.
	POP	HL		;The original value of CH_ADD is
	LD	(L5C5D),HL;CH_ADD ;restored.
	JR	L35BF;STK_PNTRS	;The subroutine exits via STK_PNTRS which
				;resets the pntrs.

